@@ -16,7 +16,6 @@ def diffutil_verify_input(input):
             _sys.exit(-1)
         _click.Path(exists=True)(input[i])
 
-
     _logging.getLogger().setLevel(_logging.INFO)
     _logging.info(f":diffutil ========== STARTING DIFFUTIL ==========")
     _logging.info(f":diffutil input1: {_os.path.abspath(input[0])}")
@@ -29,9 +28,7 @@ def diffutil_verify_status(status, passthrough):
             _logging.error(msg=f":diffutil failed. Calling sys.exit\n")
             _sys.exit(status)
         else:
-            _logging.info(
-                msg=f":diffutil failed but no sys.exit as passthrough enabled\n"
-            )
+            _logging.info(msg=f":diffutil failed but no sys.exit as passthrough enabled\n")
     else:
         _logging.info(":diffutil [ALL OK]")
 
@@ -44,7 +41,7 @@ def get_filetype(path):
     Also, strips out the underscore-appended part of the suffix, e.g. _smoothed.
     """
     basename = _os.path.basename(path)
-    suffix = basename.split(".")[1].lower().partition('_')[0]
+    suffix = basename.split(".")[1].lower().partition("_")[0]
     filetype_dict = {"snx": "sinex", "sum": "trace", "eph": "sp3"}
     if suffix in filetype_dict.keys():
         return filetype_dict[suffix]
@@ -87,7 +84,7 @@ def diffutil(ctx, input, passthrough, atol, coef, log_lvl):
         _logging.info(
             f":diffutil invoking '{filetype}' command based on the extension of the first argument of the input"
         )
-        ctx.invoke(diffutil.commands.get(filetype, None)) #else return default None
+        ctx.invoke(diffutil.commands.get(filetype, None))  # else return default None
     else:
         _logging.info(f":diffutil invoking {ctx.invoked_subcommand} command")
 
@@ -232,9 +229,7 @@ def pod(ctx):  # no coef
 
 @_click.command()
 @_click.argument("sinexpaths", required=True, nargs=-1, type=_click.Path(exists=True))
-@_click.option(
-    "-o", "--outdir", type=_click.Path(exists=True), help="output dir", default=None
-)
+@_click.option("-o", "--outdir", type=_click.Path(exists=True), help="output dir", default=None)
 def snxmap(sinexpaths, outdir):
     """Creates sinex station map html. Parses sinex SITE/ID block and create an html map.
     Expects paths to sinex files (.snx/.ssc). Can also be compressed with LZW (.Z)"""
@@ -248,9 +243,7 @@ def snxmap(sinexpaths, outdir):
 
 
 @_click.command()
-@_click.option(
-    "-s", "--sp3paths", required=True, multiple=True, type=_click.Path(exists=True)
-)
+@_click.option("-s", "--sp3paths", required=True, multiple=True, type=_click.Path(exists=True))
 @_click.option(
     "-c",
     "--clkpaths",
@@ -278,13 +271,9 @@ def sp3merge(sp3paths, clkpaths, output):
 
 
 @_click.command()
-@_click.option(
-    "-l", "--logglob", required=True, type=str, help="logs glob path (required)"
-)
+@_click.option("-l", "--logglob", required=True, type=str, help="logs glob path (required)")
 @_click.option("-r", "--rnxglob", type=str, help="rinex glob path")
-@_click.option(
-    "-o", "--output", type=str, help="rinex glob path", default="./metagather.snx"
-)
+@_click.option("-o", "--output", type=str, help="rinex glob path", default="./metagather.snx")
 @_click.option(
     "-fs",
     "--framesnx",
@@ -319,9 +308,7 @@ def sp3merge(sp3paths, clkpaths, output):
     help="number of threads to run in parallel",
     default=None,
 )
-def log2snx(
-    logglob, rnxglob, outfile, frame_snx, frame_dis, frame_psd, datetime, num_threads
-):
+def log2snx(logglob, rnxglob, outfile, frame_snx, frame_dis, frame_psd, datetime, num_threads):
     """
     IGS log files parsing utility. Globs over log files using LOGGLOB expression
      and outputs SINEX metadata file. If provided with frame and frame discontinuity files (soln),
@@ -385,21 +372,22 @@ def log2snx(
         num_threads=num_threads,
     )
 
+
 @_click.command()
-@_click.argument("trace_paths",nargs=-1,required=True, type=_click.Path(exists=True))
-@_click.option("-n","--name","db_name",default='trace2mongo',type=str,help="database name")
-def trace2mongo(trace_paths,db_name):
+@_click.argument("trace_paths", nargs=-1, required=True, type=_click.Path(exists=True))
+@_click.option("-n", "--name", "db_name", default="trace2mongo", type=str, help="database name")
+def trace2mongo(trace_paths, db_name):
     """Support bash wildcards. Could be used as:
-    
+
     trace2mongo  /data/ginan/examples/ex11/ex11-*.TRACE"""
     from pymongo import MongoClient
     from gnssanalysis import gn_io
 
-    client = MongoClient('localhost', 27017)
+    client = MongoClient("localhost", 27017)
     client.drop_database(db_name)
     mydb = client[db_name]
-    mydb.create_collection(name='States')
-    mydb.create_collection(name='Measurements')
+    mydb.create_collection(name="States")
+    mydb.create_collection(name="Measurements")
 
     # db_name = _os.path.basename(trace_path).partition('.')[0]
     for trace_path in trace_paths:
@@ -409,4 +397,4 @@ def trace2mongo(trace_paths,db_name):
         df_residuals = gn_io.trace._read_trace_residuals(trace_bytes)
 
         mydb.States.insert_many(trace.states2eda(df_states))
-        mydb.Measurements.insert_many(trace.residuals2eda(df_residuals))
\ No newline at end of file
+        mydb.Measurements.insert_many(trace.residuals2eda(df_residuals))
